# vim: set sw=4 ts=4 si et:
# Written by: Guido Socher, Copyright: GPL

# common and configuration parts fo lfwmail
package lfwmC;

sub getconfig(){
    my %cfdat;
    #---------------- start configuration, you may change this:
    # the url of the lfwmail program:
    $cfdat{'url'}='http://yourserver.org/cgi-bin/lfwmail';
    #
    # There should not be read access for normal users on 
    # $secretsdir since we store cookies and passwords there. 
    # So chmod 700 and chown apache is probably what you want to run.
    # This directory will also contain the lfwmail.log log file. This
    # log file rotates automatically.
    $cfdat{'secretsdir'}='/home/webmail/lfwmail/sec';
    #
    # temporary directory for mime messages. Must be writable for the
    # process id under which your webserver runs (probably: chown apache).
    $cfdat{'mimetmpdir'}='/home/webmail/lfwmail/tmp';
    #
    # how to convert html to plain text (/usr/bin/lynx -dump or /usr/bin/w3m -dump):
    $cfdat{'htmldecoder'}='/usr/bin/w3m -dump';
    #
    # where to connect to via pop3. localhost should be correct for
    # most of the cases:
    $cfdat{'pophost'}="localhost";
    #
    # domain name to use when sending (The stuff after the @ in the email):
    $cfdat{'domain'}="yourserver.org";
    #
    # shadow users (string to delete from uid before sending, this is a regexp,
    # this is a security feature. e-mail address and pop login can be different
    # by using this.):
    $cfdat{'shadowuid'}='_wm$';
    #
    # normally only the idential browser can send requests after authentication.
    # If you set this to 1 then you can copy the url and use it in a different
    # browser. Change this line only for debug/test purposes.
    $cfdat{'ignorebrowserid'}=0;
    #---------------- end configuration
    # do not change this:
    $cfdat{'ver'}='2.2';
    return(\%cfdat);
}
#------------- common functions
# append to a log file
sub appendlog($$){
    my $file=shift;
    my $str=shift;
    open(LOG,">>$file")||die "ERROR: can not append to log file $file, effective uid=$>";
    $str=~s/\s+/ /g;
    my @ltime = localtime;
    #return a date in yyyymmdd_hh:mm:ss format
    printf(LOG "%04d-%02d-%02d_%02d:%02d:%02d",1900+$ltime[5],$ltime[4]+1,$ltime[3],$ltime[2],$ltime[1],$ltime[0]);
    print LOG ";$str\n";
    close LOG;
    # rotate file:
    if ((-s $file) > 100000){
        rename("$file","$file.1")||die "ERROR: could not rotate log file, effective uid=$>";
    }
}
# escape certain html characters:
sub htmlescape($){
    my $str=shift;
    $str=~s/&/&amp;/g;
    $str=~s/</&lt;/g;
    $str=~s/"/&quot;/g;
    $str=~s/>/&gt;/g;
    $str;
}
sub getACU($){
    my $q=shift; # CGI query pointer
    my %d;
    $d{'action'}=$q->param('a');
    $d{'cookie'}=$q->param('c');
    $d{'uid'}=$q->param('uid');
    #security cleanup:
    $d{'uid'}=~s/[^\w\.\-\%]+//g if ($uid);
    $d{'cookie'}=~s/\W//g if ($cookie);
    $d{'action'}=~s/\W//g if ($action);
    return(\%d);
}
#
# print error messages in html:
sub htmldie_noheader($$){
    my $q=shift; # CGI query pointer
    my $msg=shift;
    print "<p>$msg</p>\n";
    print $q->end_html;
    print "\n";
    exit 0;
}
sub htmldie($$){
    my $q=shift; # CGI query pointer
    my $msg=shift;
    print $q->header(-Pragma=> 'no-cache',-type=>'text/html');
    print $q->start_html(-title=>"lfwmail web mail");
    htmldie_noheader($q,$msg);
}
# like htmldie but add a standard sentence:
sub htmldie_std($$){
    my $q=shift; # CGI query pointer
    my $msg=shift;
    my $cf=getconfig();
    my $url=$cf->{'url'};
    my $errstr=' Click below to restart<br><a href="'. $url .'?new" target="_top">'. $url ."</a>";
    htmldie($q,$msg . $errstr);
}
# write a session file and log the login event
sub writesessionfile($$$){
    my $q=shift; # CGI query pointer
    my $ud=shift;
    my $cf=shift;
    my $cookie=$ud->{'cookie'};
    my $uid=$ud->{'uid'};
    my $secretsdir=$cf->{'secretsdir'};
    # now we are authenticated, store the cookie in a
    # session file
    my $browserid;
    my $tmp;
    if ($cf->{'ignorebrowserid'}){
        $browserid="none";
    }else{
        $browserid=$q->user_agent()||'unknown_agent';
        $browserid=~s/[^\w\. ]+/ /g;
    }
    open(FF,">$secretsdir/$uid.session")||die "ERROR: can not write $secretsdir/$uid.session, effective uid=$>\n";
    print FF "${uid};${cookie};${browserid}:".$ud->{'pw'}."\n";
    close FF;
    chmod 0600,"$secretsdir/$uid.session";
    # write a log file
    lfwmC::appendlog("$secretsdir/lfwmail.log","login;$uid;$browserid");
}
# translate a cookie into password using the session file
# Takes a $ud pointer and returns one where the $ud->{'pw'} is set
sub cookie2pw($$$){
    my $q=shift; # CGI query pointer
    my $ud=shift;
    my $cf=shift;
    my $cook=$ud->{'cookie'};
    my $pw;
    my $tmp;
    my $browserid="";
    my @ldat;

    if (-f $cf->{'secretsdir'}."/".$ud->{'uid'}.".session"){
        # not more than 0.5 days old
        if ((-M $cf->{'secretsdir'}."/".$ud->{'uid'}.".session") < 0.5){
            if ($cf->{'ignorebrowserid'}){
                $browserid="none";
            }else{
                $browserid=$q->user_agent()||'unknown_agent';
                $browserid=~s/[^\w\. ]+/ /g;
            }
            $pw=undef;
            open(FF,$cf->{'secretsdir'}."/".$ud->{'uid'}.".session")||die "ERROR: can not read ".$cf->{'secretsdir'}."/".$ud->{'uid'}.".session".", effective uid=$>\n";
            $tmp=$ud->{'uid'}.";${cook};${browserid}";
            while(<FF>){
                if (/\w/){
                    chomp;
                    @ldat=split(/:/,$_,2);
                    if ($ldat[0] && $ldat[1] && ($ldat[0] eq $tmp)){
                        $pw=$ldat[1];
                    }
                }
            }
            close FF;
            if ($pw){
                $ud->{'pw'}=$pw;
                return($ud);
            }
            # everything else is error:
            htmldie_std($q,"Session data invalid, wrong id.");
        }else{
            # delete old
            unlink($cf->{'secretsdir'}."/".$ud->{'uid'}.".session");
        }
        htmldie_std($q,"Session too old.");
    }
    htmldie_std($q,"No active session found for ".$ud->{'uid'});
}

1;

